/**@file wxCamera.cpp: 
   wx aware part of camera that allows user to change resolution. */

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
 #include <wx/wx.h>
#endif

#include "camera.h"
#include "../../wxTools.h"

#include <initguid.h>		// Ensure GUID instantionalise.
DEFINE_GUID(MEDIASUBTYPE_I420, 0x30323449, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);  


/** Scan camera capabilities and select best video mode. */
void CameraInput::ChangeResolution(void)
{
TAudVidConfCaps *PAudVidConfCaps;
VIDEOINFOHEADER *pVIH;
wxArrayString ResolutionModes;
wxString Str;
HRESULT hr;
int CurrentMode;

  if(camera==NULL) return;
  if(!BuildGraph()) return;
 
  int count = sizeof(AUDIO_STREAM_CONFIG_CAPS);
  int size = sizeof(VIDEO_STREAM_CONFIG_CAPS);
  hr=camera->GetNumberOfCapabilities(&count, &size);
  if(FAILED(hr)) return;

  AM_MEDIA_TYPE* mt;
  int planes;  

  PAudVidConfCaps = (TAudVidConfCaps *)CoTaskMemAlloc(sizeof(TAudVidConfCaps));
  if(PAudVidConfCaps==NULL) return;

  CurrentMode = 0; 
  mt = NULL;
  for(int i=0; i<count; i++)
    {      
      camera->GetStreamCaps(i, &mt, (BYTE*)PAudVidConfCaps);

      planes=(8*mt->lSampleSize)/(PAudVidConfCaps->v.InputSize.cx*PAudVidConfCaps->v.InputSize.cy);      
      if(planes==12)
      {
        if(mt->subtype!=MEDIASUBTYPE_I420) Planes=8;
      } 

      if(Width==PAudVidConfCaps->v.InputSize.cx &&
         Height==PAudVidConfCaps->v.InputSize.cy &&
         planes==Planes) CurrentMode=i;

      Str.Printf(wxT("%dx%d: %d"),PAudVidConfCaps->v.InputSize.cx,PAudVidConfCaps->v.InputSize.cy,planes);
      if(planes==8) Str+=" Gray";
      if(planes==24 || planes==32) Str+=" Color";

      if(mt->subtype==MEDIASUBTYPE_CFCC) Str+=" !CFCC";
      if(mt->subtype==MEDIASUBTYPE_CLJR) Str+=" !CLJR";
      if(mt->subtype==MEDIASUBTYPE_CLPL) Str+=" !CLPL";
      if(mt->subtype==MEDIASUBTYPE_CPLA) Str+=" !CPLA";
      if(mt->subtype==MEDIASUBTYPE_DVCS) Str+=" !DVCS";
      if(mt->subtype==MEDIASUBTYPE_dvsd) Str+=" dvsd";
      if(mt->subtype==MEDIASUBTYPE_DVSD) Str+=" !DVSD";
      if(mt->subtype==MEDIASUBTYPE_dvsl) Str+=" dvsl";
      if(mt->subtype==MEDIASUBTYPE_dvsl) Str+=" dvhd";
      if(mt->subtype==MEDIASUBTYPE_I420) Str+=" I420";
      if(mt->subtype==MEDIASUBTYPE_IF09) Str+=" !IF09";
      if(mt->subtype==MEDIASUBTYPE_IJPG) Str+=" !IJPG";
      if(mt->subtype==MEDIASUBTYPE_IYUV) Str+=" !IYUV";
      if(mt->subtype==MEDIASUBTYPE_MDVF) Str+=" !MDVF";
      if(mt->subtype==MEDIASUBTYPE_MJPG) Str+=" MJPG";
      if(mt->subtype==MEDIASUBTYPE_Plum) Str+=" !Plum";
      if(mt->subtype==MEDIASUBTYPE_RGB1) Str+=" !RGB1";
      if(mt->subtype==MEDIASUBTYPE_RGB24) Str+=" RGB24";
      if(mt->subtype==MEDIASUBTYPE_RGB32) Str+=" RGB32";
      if(mt->subtype==MEDIASUBTYPE_RGB4) Str+=" !RGB4";
      if(mt->subtype==MEDIASUBTYPE_RGB565) Str+=" !RGB565";
      if(mt->subtype==MEDIASUBTYPE_RGB555) Str+=" !RGB555";
      if(mt->subtype==MEDIASUBTYPE_RGB8) Str+=" !RGB8";
      if(mt->subtype==MEDIASUBTYPE_TVMJ) Str+=" !TVMJ";
      if(mt->subtype==MEDIASUBTYPE_UYVY) Str+=" !UYVY";
      if(mt->subtype==MEDIASUBTYPE_WAKE) Str+=" !WAKE";
      if(mt->subtype==MEDIASUBTYPE_YUYV) Str+=" !YUYV";
      if(mt->subtype==MEDIASUBTYPE_YUY2) Str+=" YUY2";
      if(mt->subtype==MEDIASUBTYPE_YVYU) Str+=" !YVYU";
      if(mt->subtype==MEDIASUBTYPE_YVU9) Str+=" !YVU9";
      if(mt->subtype==MEDIASUBTYPE_YV12) Str+=" !YV12";
      if(mt->subtype==MEDIASUBTYPE_Y211) Str+=" !Y211";
      if(mt->subtype==MEDIASUBTYPE_Y411) Str+=" !Y411";
      if(mt->subtype==MEDIASUBTYPE_Y41P) Str+=" !Y41P";
      ResolutionModes.Add(Str);

      CoTaskMemFree(mt); mt=NULL;
    }

   if(ResolutionModes.GetCount()>1)
    {
    int result = wxGetSingleChoiceIndex(_("Select video mode:"),
	   _("Available Video Modes"), CurrentMode, ResolutionModes, NULL //wxFindSuitableParent()
	   );
    if(result>=0)
      {   	//setup an user chosen format
      hr = camera->GetStreamCaps(result, &mt, (BYTE*)PAudVidConfCaps);
      if(SUCCEEDED(hr))
        {        
        pVIH=(VIDEOINFOHEADER*)mt->pbFormat;
        if(FrameDuration!=0) pVIH->AvgTimePerFrame=FrameDuration; //use a same frame rate

        hr=camera->SetFormat(mt); 
        if(mt->subtype==MEDIASUBTYPE_dvsl || mt->subtype==MEDIASUBTYPE_dvsd
           || mt->subtype==MEDIASUBTYPE_dvhd
           //mt->subtype==MEDIASUBTYPE_dv25 || mt->subtype==MEDIASUBTYPE_dv50 ||
           //mt->subtype==MEDIASUBTYPE_dvh1  //proffessional types, CLSID not found.
          )  
             {
	     UseFilter();
             mt->subtype = MEDIASUBTYPE_RGB24;
	     grabber->SetMediaType(mt);
             }

        if(mt->subtype==MEDIASUBTYPE_MJPG)
            {
            UseFilterMJPG();
            mt->subtype = MEDIASUBTYPE_RGB24;
            grabber->SetMediaType(mt);
            }

        //if(mt->subtype==MEDIASUBTYPE_YUY2) mode = 1; 


        if(SUCCEEDED(hr))
          {
          Width = PAudVidConfCaps->v.InputSize.cx;
          Height = PAudVidConfCaps->v.InputSize.cy;
          Planes = (8*mt->lSampleSize)/(PAudVidConfCaps->v.InputSize.cx*PAudVidConfCaps->v.InputSize.cy);
          if(Planes<8)
            {
            Planes = pVIH->bmiHeader.biPlanes*pVIH->bmiHeader.biBitCount;
            }
          if(Planes==12)
            {
	    if(mt->subtype!=MEDIASUBTYPE_I420) Planes=8;	//12 bits means 8 + same garbage
            }
                  
          FrameDuration = pVIH->AvgTimePerFrame;
          MinFrameDuration = PAudVidConfCaps->v.MinFrameInterval;
          MaxFrameDuration = PAudVidConfCaps->v.MaxFrameInterval;

          if(pVIH->bmiHeader.biCompression!=0 && filter_base==NULL && Planes<8)             
            {            
            wxMessageBox(
               _T("Compressed camera stream is not supported, please select another mode!"),
	       _T("Camera Viewer demo"), wxOK|wxICON_EXCLAMATION);
	    }
          }
        }
      if(FAILED(hr))
        {
        wxMessageBox(
               _T("Changing camera resolution failed, try it again!"),
	       _T("Camera Viewer demo"), wxOK|wxICON_EXCLAMATION);
        }
      }
    }

  if(PAudVidConfCaps) {CoTaskMemFree(PAudVidConfCaps);PAudVidConfCaps=NULL;}
  if(mt) {CoTaskMemFree(mt);mt=NULL;}
}